from re import T
from tkinter.tix import Tree
import numpy as np
DEBUG = False

#### stack parameters ###
NUMBER_CELLS = 40
HAS_GDL = True
HAS_ENDPLATE = True
U_STACK = -1.959 * NUMBER_CELLS #antiparallel: -1.957 * NUMBER_CELLS #parallel -1.962 * NUMBER_CELLS # V
PARALLEL_FLOW = False # if flow in anode and cathode is parallel (otherwise contrary flow-directions)
INCLUDE_DRAG = True # if the effect of water dragged over the membrane should be included


# Mass flow in kg/s
MASS_H2O_IN_ANODE = 0.15 #Kg/s (per cell) # .15 Kg/s/m^2 -> assumed m2 cellarea
MASS_O2_IN_ANODE = 0.00001 #Kg/s (per cell)
MASS_H2_IN_CATHODE = 0.00001 #Kg/s (per cell)
MASS_H2O_IN_CATHODE = 0.15#.25 #Kg/s (per cell)

# Temperatures in K 
T_GAS_IN_A = 60 + 273.15 #K
T_GAS_IN_C = 60 + 273.15 #K
T_AMBIENT = 21 +273.15


# gas pressure in Pa
PRESSURE = 101325
# saturation of the membrane
LAMBDA = 22

CELLSIZE_Y_TOTAL = 2 #2 in krenz #m (hight, in flow direction; divided into mesh)
CELLSIZE_Z = .5 #.5 in krenz # im m; broadth of the cell (3rd dimension, temperature-distribution not simulated )

#thicknesses in m # ALL THICKNESSES FROM KRENZ WITH ASSUMPTION CL = 15 (reduced thickness GDL but assumed GDL conductivity for CL) 
THICKNESS_ENDPLATE = 50e-3
THICKNESS_BIPOLAR = 2e-3
THICKNESS_GDL = 4.935e-3
THICKNESS_ACL = 15e-6
THICKNESS_MEMBRANE = 100e-6
THICKNESS_CCL = 15e-6
THICKNESS_CONVECTIVE_BOUNDARY_ANODE = 50e-6
THICKNESS_CONVECTIVE_BOUNDARY_CATHODE = 50e-6

#thermal conductivities in W/(m*K) (ALL FROM KRENZ WITH CL=BIPOLAR CONDUCTIVITY)
THERMAL_CONDUCTIVITY_ENDPLATE = 15
THERMAL_CONDUCTIVITY_BIPOLAR = 15
THERMAL_CONDUCTIVITY_FLOWFIELD =THERMAL_CONDUCTIVITY_BIPOLAR/2 #ribs on half of the area
THERMAL_CONDUCTIVITY_GDL = 2.884 #max 2.884 min 2.458 
THERMAL_CONDUCTIVITY_ACL = .222 #max .222 min .164
THERMAL_CONDUCTIVITY_MEMBRANE = .324 #max .324 min.302 
THERMAL_CONDUCTIVITY_CCL = .222 #max .222 min .164
HEAT_TRANSFER_COEFF_CONVECTIVE_BOUNDARY_ANODE = 4000 # W/m^2/K (water flowing in tubes https://www.engineersedge.com/heat_transfer/convective_heat_transfer_coefficients__13378.htm)
HEAT_TRANSFER_COEFF_CONVECTIVE_BOUNDARY_CATHODE = 4000 # W/m^2/K (moderate speed air over surface https://www.engineersedge.com/heat_transfer/convective_heat_transfer_coefficients__13378.htm)
THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_ANODE = HEAT_TRANSFER_COEFF_CONVECTIVE_BOUNDARY_ANODE * THICKNESS_CONVECTIVE_BOUNDARY_ANODE # convert heat transfere coefficient into equivalent thermal conductivity
THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_CATHODE = HEAT_TRANSFER_COEFF_CONVECTIVE_BOUNDARY_CATHODE * THICKNESS_CONVECTIVE_BOUNDARY_CATHODE # convert heat transfere coefficient into equivalent thermal conductivity

THERMAL_CONDUCTIVITY_GDL_Y = 10*THERMAL_CONDUCTIVITY_GDL

ALPHA_ENDPLATE_AMBIENT = 45 # W / m^2 / K  heat transfer coefficient from endplate to ambient on lateral surface (from Krenz.2023) 


#### mesh parameters ###
COLPTS = 21 #11 #Y 1D simulation
CELLSIZE_Y = np.append(np.repeat([CELLSIZE_Y_TOTAL/(COLPTS-1)],COLPTS-1), CELLSIZE_Y_TOTAL/(COLPTS-1))# in m ; last describes the size of the virtual cells 

IS_1D = False # uses only x; set colpts to 3!


#### simulation parameters ###
INITIAL_TEMPERATURE = (T_GAS_IN_A + T_GAS_IN_C)/2
INITAL_CURRENT_DENSITY = 0.01 * 1e4 # A/m^2
MAX_ITERATIONS = 1e20
CFL_NUMBER = 6 # reduce if solution diverges (but should converge for 1 as stability criteria is then met )
TOLERANCE_TEMPERATURE = 1e-6 # Kelvin
TOLERANCE_CURRDENS = 2.0e-4 # A/m^2
# THRESHOLD_TIMESTEP_INCREASE = 3e-4
# THRESHOLD_TIMESTEP_DECREASE = 1e0



#### Result outpt ###
PRINT_TO_FILE = True # if results should be print to file 
RESULTS_DIRECTORY = "C:\\Users\\beich\\Documents\\ResultsSimulationMA\\Maximal_conductivities"
WIPE_DIRECTORY = True
WRITE_INTERVALL = 10000 # number of iterations between writing to file 

## Natural Constants #
FARRADAY = 9.648533e4 # C/mol 
R = 8.314463 # J/mol/K
MOLAR_MASS_H2O = 18e-3 # kg/mol
MOLAR_MASS_H2 = 2e-3 #kg / mol
MOLAR_MASS_O2 = 32e-3 #kg/mol

## Electrochemical parameters #
STD_PRESSURE = 101325 # Pa
STD_TEMPERATURE = 323.15 # K
ALPHA_CT_REF_A = 0.515 # reference charge transfer coefficient at 333.15K (from Krenz.2023)
ALPHA_CT_REF_C = 0.5 # reference charge transfer coefficient cathode (from Correa.2022)
I_0_A_REF = 8e-5 # Reference anodic exchange current density at 353.15K in A/m2 (from Krenz.2023)
I_0_C_REF = 32401 # Reference catho\\dic exchange current density (from Correa.2022)
E_CT_A = 510 # activation Energy charge transfer coefficient (from Krenz.2023)
E_ACD = 4300 # activation Energy anodic current density 
E_CCD = 15070 # activation Energy cathodic current density (from Correa.2022) 
K_MEM = 0.5139 # linear factor determining the membrane conductivity in Ohm*m (from Krenz.2023)
E_MEM = 1268 # activation energy membrane electric conductivity 
R_CONTACT = 4.3e-6 # Ohm*m^2 constant electric contact resistance membrane-CLs (from Krenz.2023)
#K_L = 0.023 # Mass transfer coefficient in m/s (from Trinke.2017, fitted) (Krenz used strange definition for crossover over membrane)
K_L=0.1
D_O2_H2O_G = 0.282e-4 # Diffusivity of Oxygen in water vapor at reference state in m2/s (Wu.2017)
D_O2_H2O_L = 0.24e-8 # Diffusivity of Oxygen in liquid water at reference state in m2/s (Wu.2017)
D_H2_H2O_G = 0.915e-4# Diffusivity of Hydrogen in water vapor at reference state in m2/s (Wu.2017)
D_H2_H2O_L = 1.26e-8 # Diffusivity of Hydrogen in liquid water at reference state in m2/s (Trinke.2017)
POROSITY_PTL_A = 0.60 # Porosity of the anodic PTL (Krenz.2023)
POROSITY_PTL_C = 0.60 # Porosity of the cathodic PTL (Krenz.2023)


### Calculated Parameters ###

## mesh parameters ##
THICKNESS_FLOWFIELD_ANODE = THICKNESS_BIPOLAR/3 - (2 * THICKNESS_CONVECTIVE_BOUNDARY_ANODE)
THICKNESS_FLOWFIELD_CATHODE = THICKNESS_BIPOLAR/3 - (2 * THICKNESS_CONVECTIVE_BOUNDARY_CATHODE)
THICKNESS_BIPOLAR_SOLID = THICKNESS_BIPOLAR/3
if HAS_GDL:    
    N_CELL_LAYERS = 12 # (Flowfield Anode (with boundary layers), GDL, ACL, Membrane, CCL, GDL,Flowfield Cathode (with boundary layers), BP) + Endplate + endplate right side
    __CELLSIZE_X_stack__ = [THICKNESS_CONVECTIVE_BOUNDARY_ANODE, THICKNESS_FLOWFIELD_ANODE, THICKNESS_CONVECTIVE_BOUNDARY_ANODE, THICKNESS_GDL, THICKNESS_ACL, THICKNESS_MEMBRANE, THICKNESS_CCL, THICKNESS_GDL, THICKNESS_CONVECTIVE_BOUNDARY_CATHODE, THICKNESS_FLOWFIELD_CATHODE, THICKNESS_CONVECTIVE_BOUNDARY_CATHODE, THICKNESS_BIPOLAR_SOLID]
else:
    N_CELL_LAYERS = 10 # (Flowfield Anode (with boundary layers), ACL, Membrane, CCL,Flowfield Cathode (with boundary layers), BP) + Endplate + endplate right side
    __CELLSIZE_X_stack__ = [THICKNESS_CONVECTIVE_BOUNDARY_ANODE, THICKNESS_FLOWFIELD_ANODE, THICKNESS_CONVECTIVE_BOUNDARY_ANODE, THICKNESS_ACL, THICKNESS_MEMBRANE, THICKNESS_CCL, THICKNESS_CONVECTIVE_BOUNDARY_CATHODE, THICKNESS_FLOWFIELD_CATHODE, THICKNESS_CONVECTIVE_BOUNDARY_CATHODE, THICKNESS_BIPOLAR_SOLID]

CELLAREA_SEGMENT = CELLSIZE_Y * CELLSIZE_Z


# Set thicknesses in stack direction
if HAS_ENDPLATE: 
    ROWPTS = 2 + NUMBER_CELLS * N_CELL_LAYERS + 2
else:
    ROWPTS = 1 + NUMBER_CELLS * N_CELL_LAYERS + 1

if HAS_ENDPLATE:
    CELLSIZE_X = np.array([THICKNESS_ENDPLATE, THICKNESS_BIPOLAR_SOLID])
else:    
    CELLSIZE_X = np.array([THICKNESS_BIPOLAR_SOLID])

for i in range(0,NUMBER_CELLS):
    CELLSIZE_X = np.concatenate((CELLSIZE_X,np.array(__CELLSIZE_X_stack__)))
if HAS_ENDPLATE:
    CELLSIZE_X = np.concatenate((CELLSIZE_X,[THICKNESS_ENDPLATE, np.max(CELLSIZE_X)])) #last thickness describes thickness of virtual cell
else:
    CELLSIZE_X = np.concatenate((CELLSIZE_X,[np.max(CELLSIZE_X)])) #last thickness describes thickness of virtual cell
    

## thermal paramters ##
# x direction
if HAS_GDL:
    __THERMAL_CONDUCIVITY_X_STACK__ = np.array([THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_ANODE, THERMAL_CONDUCTIVITY_FLOWFIELD, THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_ANODE, THERMAL_CONDUCTIVITY_GDL, THERMAL_CONDUCTIVITY_ACL, THERMAL_CONDUCTIVITY_MEMBRANE, THERMAL_CONDUCTIVITY_CCL, THERMAL_CONDUCTIVITY_GDL, THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_CATHODE, THERMAL_CONDUCTIVITY_FLOWFIELD, THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_CATHODE, THERMAL_CONDUCTIVITY_BIPOLAR])
else:
    __THERMAL_CONDUCIVITY_X_STACK__ = np.array([THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_ANODE, THERMAL_CONDUCTIVITY_FLOWFIELD, THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_ANODE, THERMAL_CONDUCTIVITY_ACL, THERMAL_CONDUCTIVITY_MEMBRANE, THERMAL_CONDUCTIVITY_CCL, THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_CATHODE, THERMAL_CONDUCTIVITY_FLOWFIELD, THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_CATHODE, THERMAL_CONDUCTIVITY_BIPOLAR])
if HAS_ENDPLATE:
    THERMAL_CONDUCTIVITY_X = np.array([THERMAL_CONDUCTIVITY_ENDPLATE, THERMAL_CONDUCTIVITY_BIPOLAR])
else:
    THERMAL_CONDUCTIVITY_X = np.array([THERMAL_CONDUCTIVITY_BIPOLAR, ])    
for i in range(0,NUMBER_CELLS):
    THERMAL_CONDUCTIVITY_X = np.concatenate((THERMAL_CONDUCTIVITY_X,__THERMAL_CONDUCIVITY_X_STACK__))
if HAS_ENDPLATE:
    THERMAL_CONDUCTIVITY_X = np.concatenate((THERMAL_CONDUCTIVITY_X, [THERMAL_CONDUCTIVITY_ENDPLATE, np.max(THERMAL_CONDUCTIVITY_X)])) #last thermal conductivity describes conductivity of virtual cell (not higher than highest value to not influence dt)
else:
    THERMAL_CONDUCTIVITY_X = np.concatenate((THERMAL_CONDUCTIVITY_X, [np.min(THERMAL_CONDUCTIVITY_X)])) #last thermal conductivity describes conductivity of virtual cell (not higher than highest value to not influence dt)


# y direction
if HAS_GDL:
    __THERMAL_CONDUCIVITY_Y_STACK__ = np.array([THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_ANODE, THERMAL_CONDUCTIVITY_FLOWFIELD, THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_ANODE, THERMAL_CONDUCTIVITY_GDL_Y, THERMAL_CONDUCTIVITY_ACL, THERMAL_CONDUCTIVITY_MEMBRANE, THERMAL_CONDUCTIVITY_CCL, THERMAL_CONDUCTIVITY_GDL_Y, THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_CATHODE, THERMAL_CONDUCTIVITY_FLOWFIELD, THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_CATHODE, THERMAL_CONDUCTIVITY_BIPOLAR])
else:
    __THERMAL_CONDUCIVITY_Y_STACK__ = np.array([THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_ANODE, THERMAL_CONDUCTIVITY_FLOWFIELD, THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_ANODE, THERMAL_CONDUCTIVITY_ACL, THERMAL_CONDUCTIVITY_MEMBRANE, THERMAL_CONDUCTIVITY_CCL, THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_CATHODE, THERMAL_CONDUCTIVITY_FLOWFIELD, THERMAL_CONDUCTIVITY_CONVECTIVE_BOUNDARY_CATHODE, THERMAL_CONDUCTIVITY_BIPOLAR])
if HAS_ENDPLATE:
    THERMAL_CONDUCTIVITY_Y = np.array([THERMAL_CONDUCTIVITY_ENDPLATE, THERMAL_CONDUCTIVITY_BIPOLAR,])
else:
    THERMAL_CONDUCTIVITY_Y = np.array([THERMAL_CONDUCTIVITY_BIPOLAR])
for i in range(0,NUMBER_CELLS):
    THERMAL_CONDUCTIVITY_Y = np.concatenate((THERMAL_CONDUCTIVITY_Y,__THERMAL_CONDUCIVITY_Y_STACK__))
if HAS_ENDPLATE:
    THERMAL_CONDUCTIVITY_Y = np.concatenate((THERMAL_CONDUCTIVITY_Y, [THERMAL_CONDUCTIVITY_ENDPLATE, np.max(THERMAL_CONDUCTIVITY_Y)])) #last thermal conductivity describes conductivity of virtual cell (not higher than highest value to not influence dt)
else:
    THERMAL_CONDUCTIVITY_Y = np.concatenate((THERMAL_CONDUCTIVITY_Y, [np.min(THERMAL_CONDUCTIVITY_Y)])) #last thermal conductivity describes conductivity of virtual cell (not higher than highest value to not influence dt)

## index arrays ##
# apply always on arrays with boundary cells 

if HAS_ENDPLATE:
    __endplate_add=2
else:
    __endplate_add=1
INDEX_ANODE_FLOWFIELD = range(2+__endplate_add, N_CELL_LAYERS*NUMBER_CELLS+1+__endplate_add,N_CELL_LAYERS)
INDEX_CATHODE_FLOWFIELD = range(N_CELL_LAYERS-2+__endplate_add, N_CELL_LAYERS*NUMBER_CELLS+1+__endplate_add,N_CELL_LAYERS)
INDEX_CATHODE_FLOWFIELD_BACKSIDE = range(N_CELL_LAYERS-1+__endplate_add, N_CELL_LAYERS*NUMBER_CELLS+1+__endplate_add,N_CELL_LAYERS)
INDEX_INNER_ANODE_FF_BOUNDARY_LAYER = range(3+__endplate_add, N_CELL_LAYERS*NUMBER_CELLS+1+__endplate_add,N_CELL_LAYERS)
INDEX_INNER_CATHODE_FF_BOUNDARY_LAYER = range(N_CELL_LAYERS-3+__endplate_add, N_CELL_LAYERS*NUMBER_CELLS+1+__endplate_add,N_CELL_LAYERS)
if HAS_GDL:
    INDEX_ACL = range(5+__endplate_add, N_CELL_LAYERS*NUMBER_CELLS+1+__endplate_add,N_CELL_LAYERS)
    INDEX_CCL = range(7+__endplate_add, N_CELL_LAYERS*NUMBER_CELLS+1+__endplate_add,N_CELL_LAYERS)
    INDEX_MEMBRANE = range(6+__endplate_add, N_CELL_LAYERS*NUMBER_CELLS+1+__endplate_add,N_CELL_LAYERS)
    INDEX_GDL_A = range(4+__endplate_add, N_CELL_LAYERS*NUMBER_CELLS+1+__endplate_add,N_CELL_LAYERS)
    INDEX_GDL_C = range(8+__endplate_add, N_CELL_LAYERS*NUMBER_CELLS+1+__endplate_add,N_CELL_LAYERS)
else:
    INDEX_ACL = range(4+__endplate_add, N_CELL_LAYERS*NUMBER_CELLS+1+__endplate_add,N_CELL_LAYERS)
    INDEX_CCL = range(6+__endplate_add, N_CELL_LAYERS*NUMBER_CELLS+1+__endplate_add,N_CELL_LAYERS)
    INDEX_MEMBRANE = range(5+__endplate_add, N_CELL_LAYERS*NUMBER_CELLS+1+__endplate_add,N_CELL_LAYERS)