import datetime
from tkinter.tix import Tree
import numpy as np
import sys
import os
import matplotlib.pyplot as plt
import matplotlib.animation as animation

import result_io

#set font type 
font = {'family' : 'Times New Roman',
        'weight' : 'normal',
        'size'   : 12}

plt.rc('font', **font)

### parameters ###
directory = "C:\\Users\\benedje\\OneDrive\\Studium\\Auslandssemester Trondheim\\Masters Thesis\\Results Simulation\\Krenz_parallel" # directory to read from
#directory = "C:\\Users\\beich\\Documents\\ResultsSimulationMA\\Plots\\i2"
animationName = "FluidFlowAnimation"

numPlots = 100 # how many plots to save
plotCellNr = None# None to plot all cells
discrete = False # if cells should be visible (otherwise values in between approximated)
saveOnlyLast = True # create only a plot for the final Iteration
excludeEndplates = True

#storage
wipe_directory = False
savePlots = True
saveAnimation = False
saveCrosssectionPlot = True
file_format = 'png' # fileformat to use for plots (use png, svg or pdf)

#variable to plot
plot_variable = 't_c' # variable to be plotted 
once_per_EC = False # if variable only exists once per EC in x direciton or in every cell in the mesh


#determine available iterations in directory
sorted_iterations = result_io.readResultDictory(directory)
number_of_frames = len(sorted_iterations)

#read parameters form directory
params = result_io.readParameters(directory)

if not params['HAS_ENDPLATE']:
    excludeEndplates = False

#link create folder to 
figure_dir = os.path.join(directory,f'Plots\\{plot_variable}')
result_io.makeDirectory(figure_dir,False,wipe_directory)

data, name =result_io.readFromFile(0,directory,plot_variable)

if plotCellNr is not None:
    if once_per_EC:
        xIdx = np.array([plotCellNr, plotCellNr])
    else:
        if params['HAS_ENDPLATE']:
            xIdx = range(plotCellNr*params['N_CELL_LAYERS']+1,(plotCellNr+1)*params['N_CELL_LAYERS']+3)    
        else:
            xIdx = range(plotCellNr*params['N_CELL_LAYERS'],(plotCellNr+1)*params['N_CELL_LAYERS']+2)    

else:
    if once_per_EC or not excludeEndplates:
        xIdx = range(0,np.size(data,0))
    else:
        xIdx = range(1,np.size(data,0)-1)

yIdx = range(0,np.size(data,1)) # Exclude first and last cell and here only half heat sources etc. 

#Create mesh for X and Y inputs to the figure
x=np.add.accumulate(params["CELLSIZE_X"])[0:-1]
y=np.add.accumulate(params["CELLSIZE_Y"])[0:-1]
x = np.insert(x,0, 0)
y = np.insert(y,0, 0)
if once_per_EC:
    if plotCellNr:
        x = np.array([x[params['INDEX_MEMBRANE'][plotCellNr]],x[params['INDEX_MEMBRANE'][plotCellNr]]+1e-4])
    else:
        x = x[params['INDEX_MEMBRANE']]
else:
    x = x[xIdx]
    y = y[np.array(yIdx)]

[Y,X]=np.meshgrid(y,x) #Determine indexing for stream plot (10 points only)
# X=X.transpose()
# Y=Y.transpose()
# index_cut_x=int(params['ROWPTS']/10)
# index_cut_y=int(params['COLPTS']/10)
figsize=(8.5,5)
fig =plt.figure(figsize=figsize)
fig.tight_layout()
ax=plt.axes()#Create initial ytour and stream plot as well as color bar
# if(params['IS_1D']):
#     #set all cols to value of simulated column (other columns have nonesense values as not used in simulation)
#     t_p[:,0] = t_p[:,1]
#     t_p[:,2] = t_p[:,1]
# if plotCellNr is not None:
#     ax.set_xticks(x, minor=False)
#     #ax.set_yticks(y, minor=False)
ax.yaxis.grid(True, which='major',color='k')
#     ax.xaxis.grid(True, which='major',color='k')
# ax.set_xlim([0,x[-1]])
# ax.set_ylim([0,y[-1]])
# ax.set_xlabel("$x$",fontsize=12)
# ax.set_ylabel("$y$",fontsize=12)
# ax.set_title("Iteration No: 0")
# if discrete:
#     cont=ax.pcolormesh(X,Y,t_p[xIdx,:])
# else:
#     cont=ax.contourf(X,Y,t_p[xIdx,:],levels=264)
# #stream=ax.streamplot(X[::index_cut_y,::index_cut_x],Y[::index_cut_y,::index_cut_x],u_p[::index_cut_y,::index_cut_x],v_p[::index_cut_y,::index_cut_x],color="k")
#cbar = fig.colorbar(cont)

def animate(i):    #Print frames left to be added to the animation
    sys.stdout.write("\rFrames remaining: {0:03d}".format(len(sorted_iterations)-i))
    sys.stdout.flush()    #Get iterations in a sequential manner through sorted_iterations
    iteration=sorted_iterations[i]    #Use the read_datafile function to get pressure and velocities
    data,name=result_io.readFromFile(iteration,directory,plot_variable)   #Clear previous plot and make contour and stream plots for current iteration
    if(params['IS_1D']):
        #set all cols to value of simulated column (other columns have nonesense values as not used in simulation)
        data[:,0] = data[:,1]
        data[:,2] = data[:,1]
    ax.clear()
    
    if plotCellNr is not None:
        if x.size > 1:
            ax.set_xticks(x, minor=False)
        #ax.set_yticks(y, minor=False)
        ax.xaxis.grid(True, which='major',color='k')
        #ax.yaxis.grid(True, which='major',color='k')
    if x.size > 1:
        ax.set_xlim([x[0],x[-1]])
    ax.set_ylim([y[0],y[-1]])
    ax.set_xlabel("$x [m]$",fontsize=12)
    ax.set_ylabel("$y [m]$",fontsize=12)
    #ax.set_title("Iteration No: {0}".format(i*params['WRITE_INTERVALL']))
    plt_data = data[np.ix_(xIdx,yIdx)]
    if plt_data.ndim == 1:
        plt_data = np.array([plt_data, plt_data])
    if plot_variable=="heat_flux_x" or plot_variable=="heat_flux_y":
        cont=ax.contourf(X,Y,plt_data)
        #cont=ax.contourf(X,Y,plt_data[:,:,1])
        #
        #stream = ax.streamplot(x,y,plt_data[:,:,0],plt_data[:,:,1])
    elif discrete:
        cont=ax.pcolormesh(X,Y,plt_data)  
    else:
        cont=ax.contourf(X,Y,plt_data)
    cbar = fig.colorbar(cont)
    cbar.ax.set_ylabel(name)
    # stream=ax.streamplot(X[::index_cut_y,::index_cut_x],\
    #                      Y[::index_cut_y,::index_cut_x],\
    #                      u_p[::index_cut_y,::index_cut_x],\
    #                      v_p[::index_cut_y,::index_cut_x],\
    #                      color="k")
    fig.tight_layout()
    if savePlots:
        if plotCellNr is not None:
            loc = os.path.join(figure_dir,f"plot_cellNr{plotCellNr}_{i}.{file_format}")
        else:
            loc = os.path.join(figure_dir,f"plot_{i}.{file_format}")
        fig.savefig(loc, bbox_inches="tight", dpi=300)        
    cbar.remove()

#    return cont,stream
    return cont

print("######## Making FlowPy Animation ########")
print("#########################################")
print("# Timestemp: {0}".format(datetime.datetime.now()))
if saveAnimation:
    anim=animation.FuncAnimation(fig,animate,frames=number_of_frames,interval=50,blit=False)
    movie_path=os.path.join(figure_dir,f"{animationName}.mp4")
    anim.save(r"{0}".format(movie_path))
    print(f"\nAnimation saved as {animationName}.mp4 in '{figure_dir}'")
    if(savePlots):
        print(f"\nSaved all plots to '{figure_dir}'")
elif savePlots:
    step = 1
    if number_of_frames > numPlots:
        step = int(number_of_frames / numPlots)
        
    if not saveOnlyLast:
        for i in range(number_of_frames-int(np.ceil(number_of_frames/10)),number_of_frames,step):
            animate(i)
        if number_of_frames%step != 0:
            animate(number_of_frames-1)
    else:
        animate(number_of_frames-1)

    print(f"\nSaved all plots to '{figure_dir}'")

if saveCrosssectionPlot:
    # plot xy of results
    data, name=result_io.readFromFile(sorted_iterations[-1],directory, plot_variable)
    if(params['IS_1D']):
        #set all cols to value of simulated column (other columns have nonesense values as not used in simulation)
        data[:,0] = data[:,1]
        data[:,2] = data[:,1]
    data = data[np.ix_(xIdx,yIdx)]
    #print results:
    #print(np.array_str(data,precision=2))
    #horizontal
    if True:
        fig = plt.figure(figsize=figsize,dpi=300)
        ax = plt.axes()    
        ax.yaxis.grid(True, which='major',color='k')
        plt_row = data[:,int(np.shape(data)[1]/2)]
        ax.plot(x, plt_row,'.-')
        ax.set_xlabel("x [m]")
        ax.set_ylabel(name)
        ax.set_ylim([plt_row.min()-1,plt_row.max()+1])
        if plotCellNr is not None:
            loc = os.path.join(figure_dir,f"Cross_Section_CellNr{plotCellNr}_horizontal.{file_format}")    
        else:
            loc = os.path.join(figure_dir,f"Cross_Section_horizontal.{file_format}")
        fig.savefig(loc, bbox_inches="tight")

    # vertical
    if True:
        fig = plt.figure(figsize=(16,8),dpi=300)
        plt_row = data[params["INDEX_MEMBRANE"][39]-2,:]
        ax = plt.axes()
        ax.plot(y, plt_row,'.-')
        ax.yaxis.grid(True, which='major',color='k')
        plt.xlabel("y")
        plt.ylabel(name)
        #plt.ylim([.125,.175])
        if plotCellNr is not None:
            loc = os.path.join(figure_dir,f"Cross_Section_CellNr{plotCellNr}_vertical.{file_format}")    
        else:
            loc = os.path.join(figure_dir,f"Cross_Section_vertical_cell39.{file_format}")
        fig.savefig(loc, bbox_inches="tight")

    #plot multiple results
    if False:
        directory="C:\\Users\\benedje\\Documents\\Python simulation\\Results\\max_cond_antiparallel"    
        sorted_iterations = result_io.readResultDictory(directory)
        data, name=result_io.readFromFile(sorted_iterations[-1],directory, plot_variable)
        data = data[np.ix_(xIdx,yIdx)]   
        data = np.expand_dims(data,2)
        directory="C:\\Users\\benedje\\Documents\\Python simulation\\Results\\max_cond_parallel"
        sorted_iterations = result_io.readResultDictory(directory)
        data2,_ = result_io.readFromFile(sorted_iterations[-1],directory, plot_variable)       
        data2 = data2[np.ix_(xIdx,yIdx)]
        data2 = np.expand_dims(data2,2)        
        data = np.append(data,data2,2)
        # directory="C:\\Users\\beich\\Documents\\ResultsSimulationMA\\sinter3"
        # sorted_iterations = result_io.readResultDictory(directory)
        # data3,_ = result_io.readFromFile(sorted_iterations[-1],directory, plot_variable)
        # data3 = data3[np.ix_(xIdx,yIdx)]
        # data3 = np.expand_dims(data3,2)
        # data = np.append(data,data3,2)

        #print results:
        #print(np.array_str(data,precision=2))
        #horizontal
        fig = plt.figure(figsize=figsize,dpi=300)
        ax = plt.axes()    
        ax.yaxis.grid(True, which='both')
        plt_row = data[:,int(np.shape(data)[1]/2)]
        plt.plot(x, plt_row,'.-')
        plt.legend(["2.4675 mm PTL", "4.935 mm PTL"])
        plt.xlabel("x [m]")
        plt.ylabel(name)
        plt.ylim([plt_row.min()-1,plt_row.max()+1])
        if plotCellNr is not None:
            loc = os.path.join(figure_dir,f"Cross_Section_CellNr{plotCellNr}_horizontal_combined.{file_format}")    
        else:
            loc = os.path.join(figure_dir,f"Cross_Section_horizontal_combined.{file_format}")
        fig.savefig(loc, bbox_inches="tight")

    # Compare vertical
    if False:
        directory="C:\\Users\\benedje\\Documents\\Python simulation\\Results\\max_cond_antiparallel"    
        sorted_iterations = result_io.readResultDictory(directory)
        data, name=result_io.readFromFile(sorted_iterations[-1],directory, plot_variable)
        # data = data[:,:,1]
        data = data[np.ix_(xIdx,yIdx)]   
        data = np.expand_dims(data,2)
        directory="C:\\Users\\benedje\\Documents\\Python simulation\\Results\\max_cond_parallel"
        sorted_iterations = result_io.readResultDictory(directory)
        data2,_ = result_io.readFromFile(sorted_iterations[-1],directory, plot_variable)       
        # data2 = data2[:,:,1]
        data2 = data2[np.ix_(xIdx,yIdx)]
        data2 = np.expand_dims(data2,2)        
        data = np.append(data,data2,2)
        # directory="C:\\Users\\beich\\Documents\\ResultsSimulationMA\\sinter3"
        # sorted_iterations = result_io.readResultDictory(directory)
        # data3,_ = result_io.readFromFile(sorted_iterations[-1],directory, plot_variable)
        # data3 = data3[np.ix_(xIdx,yIdx)]
        # data3 = np.expand_dims(data3,2)
        # data = np.append(data,data3,2)

        #print results:
        #print(np.array_str(data,precision=2))
        #vertical
        fig = plt.figure(figsize=figsize,dpi=300)
        ax = plt.axes()    
        ax.yaxis.grid(True, which='both')
        #plt_row = data[params["INDEX_MEMBRANE"][20]-2,:]
        plt_row = data[0,:,:]
        plt.plot(y, plt_row,'.-')
        plt.legend(["Counter-flow", "Parallel flow"])
        plt.xlabel("y [m]")
        plt.ylabel(name)
        if plotCellNr is not None:
            loc = os.path.join(figure_dir,f"Cross_Section_CellNr{plotCellNr}_vetical_combined.{file_format}")    
        else:
            loc = os.path.join(figure_dir,f"Cross_Section_vertical_combined.{file_format}")
        fig.savefig(loc, bbox_inches="tight")


    #plot maximal value in cell over stack
    if True:
        data, name=result_io.readFromFile(sorted_iterations[-1],directory, plot_variable)
        data = data[params["INDEX_ANODE_FLOWFIELD"][0]-2:params["INDEX_CATHODE_FLOWFIELD"][-1]+2,yIdx]
        data_cells = np.zeros((int(np.shape(data)[0]/params["N_CELL_LAYERS"])))
        for i in range(0,int(np.shape(data)[0]/params["N_CELL_LAYERS"])):
            data_cells[i] = np.max(data[i*params["N_CELL_LAYERS"]:(i+1)*params["N_CELL_LAYERS"],:])

        directory="C:\\Users\\benedje\\OneDrive\\Studium\\Auslandssemester Trondheim\\Masters Thesis\\Results Simulation\\max_cond_parallel"    
        sorted_iterations = result_io.readResultDictory(directory)
        data, name=result_io.readFromFile(sorted_iterations[-1],directory, plot_variable)
        data = data[np.ix_(xIdx,yIdx)]   
        data = data[params["INDEX_ANODE_FLOWFIELD"][0]-2:params["INDEX_CATHODE_FLOWFIELD"][-1]+2,yIdx]
        data_cells2 = np.zeros((int(np.shape(data)[0]/params["N_CELL_LAYERS"])))
        for i in range(0,int(np.shape(data)[0]/params["N_CELL_LAYERS"])):
            data_cells2[i] = np.max(data[i*params["N_CELL_LAYERS"]:(i+1)*params["N_CELL_LAYERS"],:])

        data_cells = np.expand_dims(data_cells,1)
        data_cells2 = np.expand_dims(data_cells2,1)  
        data_cells = np.append(data_cells,data_cells2,1)
    
        #print results:
        #print(np.array_str(data,precision=2))
        #horizontal
        fig = plt.figure(figsize=figsize,dpi=300)
        ax = plt.axes()    
        ax.yaxis.grid(True, which='both')
        ax.plot(range(0,len(data_cells)), data_cells,'.-')
        plt.legend(["Sinter PTL", "Felt PTL"])
        plt.xlabel("Cell Nr.")
        plt.ylabel(f"Maximal in cell: {name}")
        loc = os.path.join(figure_dir,f"Max_inCell.{file_format}")
        fig.savefig(loc, bbox_inches="tight")

        #plot mean value in cell over stack
    if False:
        data, name=result_io.readFromFile(sorted_iterations[-1],directory, plot_variable)
        data = data[params["INDEX_ANODE_FLOWFIELD"][0]-2:params["INDEX_CATHODE_FLOWFIELD"][-1]+2,yIdx]
        data_cells = np.zeros((int(np.shape(data)[0]/params["N_CELL_LAYERS"])))
        for i in range(0,int(np.shape(data)[0]/params["N_CELL_LAYERS"])):
            data_cells[i] = np.mean(data[i*params["N_CELL_LAYERS"]:(i+1)*params["N_CELL_LAYERS"],:])

    
        #print results:
        #print(np.array_str(data,precision=2))
        #horizontal
        fig = plt.figure(figsize=figsize,dpi=300)

        plt.plot(range(0,len(data_cells)), data_cells,'.-')
        plt.xlabel("Cell Nr.")
        plt.ylabel(f"Mean in cell: {name}")
        loc = os.path.join(figure_dir,f"Mean_inCell.{file_format}")
        fig.savefig(loc, bbox_inches="tight")

        #plot min value in cell over stack
    if False:
        data, name=result_io.readFromFile(sorted_iterations[-1],directory, plot_variable)
        data = data[params["INDEX_ANODE_FLOWFIELD"][0]-2:params["INDEX_CATHODE_FLOWFIELD"][-1]+2,yIdx]
        data_cells = np.zeros((int(np.shape(data)[0]/params["N_CELL_LAYERS"])))
        for i in range(0,int(np.shape(data)[0]/params["N_CELL_LAYERS"])):
            data_cells[i] = np.min(data[i*params["N_CELL_LAYERS"]:(i+1)*params["N_CELL_LAYERS"],:])

    
        #print results:
        #print(np.array_str(data,precision=2))
        #horizontal
        fig = plt.figure(figsize=figsize,dpi=300)

        plt.plot(range(0,len(data_cells)), data_cells,'.-')
        plt.xlabel("Cell Nr.")
        plt.ylabel(f"Minimal in cell: {name}")
        loc = os.path.join(figure_dir,f"Min_inCell.{file_format}")
        fig.savefig(loc, bbox_inches="tight")

        